<?php

namespace Erp\Master\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Erp\Master\Forms\Student as StudentForm;
use Erp\Master\Http\Requests\StudentResourceRequest;
use Erp\Master\Http\Resources\StudentResource;
use Erp\Master\Http\Resources\StudentsCollection;
use Erp\Master\Models\Student;
use Erp\Master\Actions\StudentAction;
use Erp\Master\Actions\StudentActions;


/**
 * Resource controller class for student.
 */
class StudentResourceController extends BaseController
{

    /**
     * Initialize student resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = StudentForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('erp.master.modules'), 'master', guard_url('master'));
            return $next($request);
        });
    }

    /**
     * Display a list of student.
     *
     * @return Response
     */
    public function index(StudentResourceRequest $request)
    {
        $request = $request->all();
        $page = StudentActions::run('paginate', $request);

        $data = new StudentsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('master::student.names'))
            ->view('master::student.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display student.
     *
     * @param Request $request
     * @param Model   $student
     *
     * @return Response
     */
    public function show(StudentResourceRequest $request, Student $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new StudentResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('master::student.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('master::student.show')
            ->output();
    }

    /**
     * Show the form for creating a new student.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(StudentResourceRequest $request, Student $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new StudentResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('master::student.name'))
            ->view('master::student.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new student.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StudentResourceRequest $request, Student $model)
    {
        try {
            $request = $request->all();
            $model = StudentAction::run('store', $model, $request);
            $data = new StudentResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('master::student.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('master/student/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/master/student'))
                ->redirect();
        }

    }

    /**
     * Show student for editing.
     *
     * @param Request $request
     * @param Model   $student
     *
     * @return Response
     */
    public function edit(StudentResourceRequest $request, Student $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new StudentResource($model);
        // return view('master::student.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('master::student.name'))
            ->view('master::student.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the student.
     *
     * @param Request $request
     * @param Model   $student
     *
     * @return Response
     */
    public function update(StudentResourceRequest $request, Student $model)
    {
        try {
            $request = $request->all();
            $model = StudentAction::run('update', $model, $request);
            $data = new StudentResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('master::student.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('master/student/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/student/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the student.
     *
     * @param Model   $student
     *
     * @return Response
     */
    public function destroy(StudentResourceRequest $request, Student $model)
    {
        try {

            $request = $request->all();
            $model = StudentAction::run('destroy', $model, $request);
            $data = new StudentResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::student.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('master/student/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/student/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
